/* ---------------------------------------------------------------------------------------------------------------------------- */
/* CREATE SVR ROLES OpIRoleSvrSuperFixed, OpIRoleSvrManagerFixed, OpIRoleSvrDataOwnerFixed, OpIRoleSvrClientFixed
	FOR SERVER INSTANCE <SERVERNAME>\<INSTANCENAME> FOR ASSIGNMENT OF PERMISSIONS TO EACH SERVER LOGIN */
/* ---------------------------------------------------------------------------------------------------------------------------- */

/* Security Architecture:  
	https://learn.microsoft.com/en-us/sql/relational-databases/security/authentication-access/getting-started-with-database-engine-permissions?view=sql-server-ver16
	https://www.mssqltips.com/sqlservertip/2699/sql-server-user-defined-server-roles/ */

USE master;
GO
/* Grant EXECUTE to OLEDB_Provider */
GRANT EXECUTE ON SYS.XP_PROP_OLEDB_PROVIDER TO [Public];
GO

USE master;	-- For Server Level Roles, always master DB
GO

/* For Server, create Roles OpIRoleSvrSuperFixed, OpIRoleSvrManagerFixed, OpIRoleSvrDataOwnerFixed - No Svr Capabilities for OpIRoleSvrClientFixed */
/* SET UP ONCE ONLY FOR THE SERVER INSTANCE <INSTANCENAME> */

/* SET UP OpIRoleSvrSuperFixed */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrSuperFixed'
SET		@activeNow = N'No'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow=N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE sysadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
	END
	
	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type]'
	+ N'	FROM [sys].[server_principals] '
	+ N'	WHERE [type] IN (N''R'') AND [name] = '''	+ @roleName	+ N''')'
	+ N'	CREATE SERVER ROLE '	+ @roleName	+ N' AUTHORIZATION securityadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	-- Cannot assign members to sysadmim...   SET @query = N'ALTER SERVER ROLE sysadmin '				+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE serveradmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE securityadmin '		+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE setupadmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE dbcreator '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
END
GO


/* SET UP OpIRoleSvrManagerFixed */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrManagerFixed'
SET		@activeNow = N'No'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow=N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE sysadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
	END
	
	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type]'
	+ N'	FROM [sys].[server_principals] '
	+ N'	WHERE [type] IN (N''R'') AND [name] = '''	+ @roleName	+ N''')'
	+ N'	CREATE SERVER ROLE '	+ @roleName	+ N' AUTHORIZATION securityadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	SET @query = N'ALTER SERVER ROLE serveradmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE securityadmin '		+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE setupadmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE dbcreator '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
END
GO


/* SET UP OpIRoleSvrDataOwnerFixed */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrDataOwnerFixed'
SET		@activeNow = N'No'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow=N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE sysadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
	END
	
	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type]'
	+ N'	FROM [sys].[server_principals] '
	+ N'	WHERE [type] IN (N''R'') AND [name] = '''	+ @roleName	+ N''')'
	+ N'	CREATE SERVER ROLE '	+ @roleName	+ N' AUTHORIZATION securityadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	SET @query = N'ALTER SERVER ROLE serveradmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE setupadmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
	SET @query = N'ALTER SERVER ROLE dbcreator '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
END
GO


/* SET UP OpIRoleSvrClientFixed */
DECLARE	@roleName NVARCHAR(max) = NULL
DECLARE	@activeNow NVARCHAR(6) = N'No'
DECLARE @replaceNow NVARCHAR(3) = N'No'
DECLARE	@query NVARCHAR(max) = NULL
SET		@roleName=N'OpIRoleSvrClientFixed'
SET		@activeNow = N'No'		-- Yes or No
SET		@replaceNow = N'No'		-- Yes or No

IF		@activeNow = N'Yes'
BEGIN
	IF	@replaceNow=N'Yes'
	BEGIN
		/* DROP previous membership inServer Fixed Roles */
		SET @query = N'ALTER SERVER ROLE sysadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE serveradmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE securityadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE processadmin	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE setupadmin 	'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE bulkadmin		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE diskadmin 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
		SET @query = N'ALTER SERVER ROLE dbcreator 		'	+ N'DROP MEMBER '	+ @roleName + N';';				EXEC sp_executesql @query;
	END
	
	SET @query = N'IF NOT EXISTS (SELECT [sys].[server_principals].[name], [sys].[server_principals].[type]'
	+ N'	FROM [sys].[server_principals] '
	+ N'	WHERE [type] IN (N''R'') AND [name] = '''	+ @roleName	+ N''')'
	+ N'	CREATE SERVER ROLE '	+ @roleName	+ N' AUTHORIZATION securityadmin;';	EXEC sp_executesql @query;
	
	/* Configure server-level permissions to the role using GRANT, DENY, REVOKE:  https://learn.microsoft.com/en-us/sql/t-sql/statements/create-server-role-transact-sql?view=sql-server-ver16 */
	-- No Server permissions for OpIRoleSvrClientFixed
	-- SET @query = N'ALTER SERVER ROLE serveradmin '			+ N'ADD MEMBER '	+ @roleName + N';';	EXEC sp_executesql @query;
END
GO